/***************************************************************************

     File: callbacks.c
 Contents: Callback routines for the xmtutor application
  Part of: The Xmtutor application
   Author: Jan Borchers (job@ira.uka.de)

  $Source: /home/mit1/job/distribution/Xmtut/RCS/callbacks.c,v $
$Revision: 1.3 $
    $Date: 1994/10/17 18:24:12 $
   $State: Rel $
  $Locker:  $

$Log: callbacks.c,v $
 * Revision 1.3  1994/10/17  18:24:12  job
 * First revision under version control.
 *

***************************************************************************/


/**************************************************************************/
/*                                                                        */
/* COPYRIGHT NOTICE:                                                      */
/* -----------------                                                      */
/* All parts of the Xmtutor application and the sample Motif tutorial are */
/* Copyright (C) 1993,1994 Jan Borchers.                                  */
/*                                                                        */
/**************************************************************************/


#include <stdio.h>						/* For debugging printf's etc. */
#include <string.h>						/* For strcmp() in ChangeSectionCB() */
#include <X11/Intrinsic.h>					/* Xt structures & variables */
#include <X11/StringDefs.h>					/* Standard Xt string constant definitions */
#include <Xm/Xm.h>						/* Common Motif variables */
#include <Xm/XmP.h>		                                /* Private Motif variables for ups debugging&XtResizeWidget */
#include <Xm/MessageB.h>					/* For ErrorDialog */
#include <Xm/SelectioB.h>					/* For SelectionDialog */
#include <Xm/Text.h>						/* For textwindow updating */
#include <Xm/List.h>						/* For XmListSelectItem() in SetSectionListCB() */

#include "EXPORT.H"
#include "callbacks.h"

#include "IMPORT.H"
#include "widgets.h"
#include "sectionio.h"
#include "runio.h"


void RestoreCB (Widget w, caddr_t client_data, caddr_t call_data)	/* Callback to restore original version of */
{									/* resource file, e.g. pushbutton.rsc */
  /* printf ("Restoring resource...\n"); */
  RestoreResource ();
}


void ExitCB (Widget w, caddr_t client_data, caddr_t call_data)          /* Callback to popup exit question dialog */
{
  XtManageChild (exitdialog);
}


void ExitOKCB (Widget w, caddr_t client_data, caddr_t call_data)	/* Callback to exit the application */
{
  /* printf ("Exit xmtutor.\n"); */
  exit(0);
}


void NextSectionCB (Widget w, caddr_t client_data, caddr_t call_data)	/* Callback to go to the next section */
     
{
  /* printf ("Next section...\n"); */

  if (currentSection < chapterInfo [currentChapter].numberOfSections - 1)
    {
      currentSection++;
      ReadSection ();
    }
  else if (currentChapter < numberOfChapters - 1 &&
	   chapterInfo [currentChapter+1].numberOfSections > 0) /* Wrap over to next chapter */
    {
      currentChapter++;
      currentSection = 0;
      ReadSection ();
    }
  else								/* Display error dialog if trying to go past last section */
    XtManageChild (lastsectionerror);				/* of the tutorial */ 
}


void PreviousSectionCB (Widget w, caddr_t client_data, caddr_t call_data) /* Callback to go to the previous section */

{
  /* printf ("Previous section...\n"); */
  if (currentSection > 0)
    {
      SaveResource ();
      currentSection--;
      ReadSection ();
    }
  else if (currentChapter > 0 && chapterInfo [currentChapter-1].numberOfSections > 0)  /* Wrap over to previous chapter */
    {
      SaveResource ();
      currentChapter--;
      currentSection = chapterInfo [currentChapter].numberOfSections - 1;
      ReadSection ();
    }
  else								/* Display error dialog if trying to go past first section */
    XtManageChild (firstsectionerror);				/* of the tutorial */
}


void SetSectionListCB (Widget w, caddr_t client_data,		/* Callback to set the list of sections in the */
		       XmListCallbackStruct *call_data)		/* "Choose a section..." dialog */

{
  XmString *sections;						/* Array of section titles as compound strings */
  XmString highlightedSections [1];				/* Current or first section - to highlight it in the list */
  String chapterTitle;
  Cardinal tempCurrentChapter, i;
  
  /* printf ("Display section titles of current chapter...\n"); */

  if (!XmStringEmpty (call_data->item))				/* Act only if a chapter has been selected */
    {
      XmStringGetLtoR (call_data->item, XmSTRING_DEFAULT_CHARSET, &chapterTitle);
      tempCurrentChapter = 0;					/* Find index of chapter title */
      while (strcmp (chapterTitle, chapterInfo [tempCurrentChapter].title) != 0)
	tempCurrentChapter++;
      
      if (chapterInfo[tempCurrentChapter].numberOfSections > 0)	/* Show sections only if there are any */
	{
	  sections = (XmString *) XtMalloc (sizeof (XmString) * chapterInfo[tempCurrentChapter].numberOfSections);
	  for (i = 0; i < chapterInfo [tempCurrentChapter].numberOfSections; i++)
	    sections [i] = XmStringCreate (sectionInfo [chapterInfo [tempCurrentChapter].sectionOffset+i].title,
					   XmSTRING_DEFAULT_CHARSET);
	  if (tempCurrentChapter == currentChapter)
	    highlightedSections [0] = XmStringCopy (sections [currentSection]);
	  else
	    highlightedSections [0] = XmStringCopy (sections [0]);
	  
	  XtVaSetValues (choosesectionlist,
			 XmNitems, sections,
			 XmNitemCount, chapterInfo[tempCurrentChapter].numberOfSections,
			 XmNselectedItems, highlightedSections,
			 XmNselectedItemCount, 1,
			 NULL);
	  XmListSelectItem (choosesectionlist, highlightedSections[0], TRUE);
	  for (i = 0; i < chapterInfo [tempCurrentChapter].numberOfSections; i++)
	    XmStringFree (sections [i]);
	  XtFree ((char *) sections);
	}
    }
  XtResizeWidget (XtParent(choosesectiondialog), 640,400,0); 
  XtResizeWindow (XtParent(choosesectiondialog));
}


void ChooseSectionCB (Widget w, caddr_t client_data, caddr_t call_data) /* Callback to choose a chapter and section */

{
  XmString *chapters;						/* Array of chapter titles as compound strings */
  XmString highlightedChapters [1];				/* Current chapter - to highlight it in the list */
  Cardinal i;
  
  /* printf ("Choose a chapter & section...\n"); */
  
  chapters = (XmString *) XtMalloc (sizeof (XmString) * numberOfChapters);

  for (i = 0; i < numberOfChapters; i++)
    chapters [i] = XmStringCreate (chapterInfo [i].title, XmSTRING_DEFAULT_CHARSET);

  highlightedChapters [0] = XmStringCopy (chapters [currentChapter]);

  XtVaSetValues (choosechapterlist,
		 XmNitems, chapters,
		 XmNitemCount, numberOfChapters,
		 XmNselectedItems, highlightedChapters,
		 XmNselectedItemCount, 1,
		 NULL);
  XmListSelectItem (choosechapterlist, highlightedChapters[0], TRUE);

  XmStringFree (highlightedChapters [0]);
  for (i = 0; i < numberOfChapters; i++)
    XmStringFree (chapters [i]);
  XtFree ((char *) chapters);

  XtManageChild (choosesectiondialog);
  XtResizeWidget (XtParent(choosesectiondialog), 640,400,0); 
  XtResizeWindow (XtParent(choosesectiondialog));
}


void ChooseOKCB (Widget w, caddr_t client_data, caddr_t call_data)  /* CB to change to chosen section */
								    /* Section is sure to exist */
{
  XmStringTable selectedChapters, selectedSections;
  String chapterTitle, sectionTitle;
  Cardinal index, n1, n2;
  
  XtVaGetValues (choosechapterlist,
		 XmNselectedItems, &selectedChapters,
		 XmNselectedItemCount, &n1,
		 NULL);

  XtVaGetValues (choosesectionlist,
		 XmNselectedItems, &selectedSections,
		 XmNselectedItemCount, &n2,
		 NULL);

  if (!XmStringEmpty (selectedChapters [0]))
    {
      SaveResource ();
      XmStringGetLtoR (selectedChapters [0], XmSTRING_DEFAULT_CHARSET, &chapterTitle);
      index = 0;						/* Find index of chapter title */
      while (strcmp (chapterTitle, chapterInfo [index].title) != 0)
	index++;
      currentChapter = index;

      XmStringGetLtoR (selectedSections [0], XmSTRING_DEFAULT_CHARSET, &sectionTitle);
      index = 0;						/* Find index of section title */
      while (strcmp (sectionTitle, sectionInfo [chapterInfo [currentChapter].sectionOffset + index].title) != 0)
	index++;
      currentSection = index;

      /* printf ("Chosen: Chapter %u = %s, Section %u = %s.\n", currentChapter, chapterTitle, currentSection, sectionTitle); */
      ReadSection ();
    }
  XtUnmanageChild (choosesectiondialog);			/* Pop down choosing dialog */
}


void ChooseCancelCB (Widget w, caddr_t client_data, caddr_t call_data)		 /* CB to cancel section choosing */
{
  XtUnmanageChild (choosesectiondialog);
}


void IndexCB (Widget w, caddr_t client_data, caddr_t call_data)		 /* CB to display index list */
{
  XtManageChild (indexdialog);
}


void IndexOKCB (Widget w, caddr_t client_data, XmSelectionBoxCallbackStruct *call_data) /* CB to jump to section of
											   chosen keyword */
{
  String selection;
  Cardinal i;
  
  XtUnmanageChild (indexdialog);
  XmStringGetLtoR (call_data->value, XmSTRING_DEFAULT_CHARSET, &selection);
  i = 0;
  while (strcmp(selection, indexEntries [i].keyword) != 0)         /* keyword is guaranteed to exist because of XmNmustMatch */
    i++;
  currentChapter = indexEntries [i].chapter;
  currentSection = indexEntries [i].section;
  ReadSection();
}


void IndexCancelCB (Widget w, caddr_t client_data, XmSelectionBoxCallbackStruct *call_data) /* CB to cancel index dialog */

{
  XtUnmanageChild (indexdialog);
}


void IndexNoMatchCB (Widget w, caddr_t client_data, caddr_t call_data)		 /* CB if index keyword does not exist */

{
  XtManageChild (nosuchindexerror);
}


void RunCB (Widget w, caddr_t client_data, caddr_t call_data)		 /* CB to run example program */

{
  /* printf ("Run example program...\n"); */
  SaveResource();
  RunExample();
}


void OnMenusCB (Widget w, caddr_t client_data, caddr_t call_data)	 /* CB to display help on menus */

{
  /* printf ("Display help on menus.\n"); */
  XtManageChild (onmenusdialog);
}


void OnVersionCB (Widget w, caddr_t client_data, caddr_t call_data)	 /* CB to display help on version */

{
  /* printf ("Display help on version.\n"); */
  XtManageChild (onversiondialog);
}
