/***************************************************************************

     File: init.c
 Contents: Getting Xmtutor resources, Initializing index
  Part of: The Xmtutor application
   Author: Jan Borchers (job@ira.uka.de)

  $Source: /home/mit1/job/distribution/Xmtut/RCS/init.c,v $
$Revision: 1.3 $
    $Date: 1994/10/17 18:07:19 $
   $State: Rel $
  $Locker:  $

$Log: init.c,v $
 * Revision 1.3  1994/10/17  18:07:19  job
 * First revision under version control.
 *

***************************************************************************/


/**************************************************************************/
/*                                                                        */
/* COPYRIGHT NOTICE:                                                      */
/* -----------------                                                      */
/* All parts of the Xmtutor application and the sample Motif tutorial are */
/* Copyright (C) 1993,1994 Jan Borchers.                                  */
/*                                                                        */
/**************************************************************************/


#include <stdio.h>						/* Debugging printf's etc. */
#include <string.h>						/* C string functions */
#include <X11/Intrinsic.h>					/* Xt structures, variables & functions */
#include <X11/StringDefs.h>					/* Standard Xt string constant definitions */
#include <Xm/Xm.h>						/* Common Motif variables */

#include "IMPORT.H"
#include "sectionio.h"						/* For ReadIndexFile () */

#include "EXPORT.H"
#include "init.h"


static void CutSpaces (String s)				/* Remove trailing spaces from s, if any */

{
  int i;

  i = strlen (s) - 1;						/* Start with last character in s */
  while ((i >= 0) && (s[i] == ' '))				/* While it is a space and s is not empty: */
    s [i--] = '\0';						/* Replace with string terminator */
}


void GetXmtutorResources (Widget toplevel)			/* Gets application resources for Xmtutor, especially */
								/* chapter information */
{
  Cardinal i, j, k;
  String resource_name;
  
  XtResource resources [] =					/* Get the number of chapters from the resource file */
    {
      {"numberOfChapters", "NumberOfChapters", XtRCardinal,
	 sizeof (Cardinal), (Cardinal) 0, XtRImmediate, (caddr_t) 0}
    };
  
  XtGetApplicationResources (toplevel, &numberOfChapters, resources, XtNumber (resources), NULL, 0);
  /* printf ("Number of chapters: %d\n",numberOfChapters); */

  resource_name = XtNewString ("chapter_XXX_directory");	/* Reserve memory for longest possible resource name once */

  for (i=0; i<numberOfChapters; i++)				
    {								
      resources [0].resource_class = "ChapterTitle";		/* Retrieve chapter titles */
      resources [0].resource_type = XtRString;			
      resources [0].resource_size = sizeof (String);		/* resources[] must be initialized repeatedly */
      resources [0].resource_offset = 0;			/* since XtGetApplicationResources is destructive */
      resources [0].default_type = XtRImmediate;
      resources [0].default_addr = (caddr_t) "";
      sprintf (resource_name,"chapter_%d_title", i);		/* Construct resource name, e.g. "chapter_2_title" */
      resources [0].resource_name = resource_name;
      chapterInfo [i].title = (String) XtMalloc (80*sizeof(char));
      chapterInfo [i].title = "Noch nix...";
      XtGetApplicationResources (toplevel, &(chapterInfo [i].title), resources, XtNumber (resources), NULL, 0);
      CutSpaces (chapterInfo[i].title);				/* Remove trailing spaces, if any */
      /* printf ("Chapter %d: Title >%s<, ", i, chapterInfo [i].title); */

      resources [0].resource_class = "ChapterDirectory";	/* Retrieve chapter directories */
      resources [0].resource_type = XtRString;			
      resources [0].resource_size = sizeof (String);
      resources [0].resource_offset = 0;
      resources [0].default_type = XtRImmediate;
      resources [0].default_addr = (caddr_t) "";
      sprintf (resource_name,"chapter_%d_directory", i);	/* Construct resource name, e.g. "chapter_2_directory" */
      resources [0].resource_name = resource_name;
      chapterInfo [i].directory = (String) XtMalloc (80*sizeof(char));
      chapterInfo [i].directory = "Noch nix...";
      XtGetApplicationResources (toplevel, &(chapterInfo [i].directory), resources, XtNumber (resources), NULL, 0);
      CutSpaces (chapterInfo[i].directory);			/* Remove trailing spaces, if any */
      /* printf ("Directory >%s<, ", chapterInfo [i].directory); */

      resources [0].resource_class = "NumberOfSections";	/* Retrieve number of sections in current chapter */
      resources [0].resource_type = XtRCardinal;		
      resources [0].resource_size = sizeof (Cardinal);
      resources [0].resource_offset = 0;
      resources [0].default_type = XtRImmediate;
      resources [0].default_addr = (caddr_t) 0;
      chapterInfo [i].numberOfSections = 42;
      sprintf (resource_name,"chapter_%d_numberOfSections", i);	/* Construct resource name,e.g. "chapter_2_numberOfSections" */
      resources [0].resource_name = resource_name;
      XtGetApplicationResources (toplevel, &(chapterInfo [i].numberOfSections), resources, XtNumber (resources), NULL, 0);     
      /* printf ("Sections >%u<, ", chapterInfo [i].numberOfSections); */
      if (i == 0)						/* Compute offset for index of sections in this chapter */
	chapterInfo [i].sectionOffset = 0;
      else
	chapterInfo [i].sectionOffset = chapterInfo [i-1].sectionOffset + chapterInfo [i-1].numberOfSections;
      /* printf ("Offset >%u<.\n", chapterInfo [i].sectionOffset); */
    };
  /* printf ("Key..."); getchar (); */

  for (i = 0; i < numberOfChapters; i++)				/* Retrieve section information */
    for (j = 0; j < chapterInfo [i].numberOfSections; j++)
      {								
	k = chapterInfo[i].sectionOffset + j;
	resources [0].resource_class = "SectionTitle";			/* Retrieve section titles */
	resources [0].resource_type = XtRString;			
	resources [0].resource_size = sizeof (String);			/* resources[] must be initialized repeatedly */
	resources [0].resource_offset = 0;				/* since XtGetApplicationResources is destructive */
	resources [0].default_type = XtRImmediate;
	resources [0].default_addr = (caddr_t) "";
	sprintf (resource_name,"section_%d_%d_title", i, j);		/* Construct resource name, e.g. "section_2_0_title" */
	resources [0].resource_name = resource_name;
	sectionInfo [k].title = (String) XtMalloc (80*sizeof(char));
	sectionInfo [k].title = "Noch nix...";
	XtGetApplicationResources (toplevel, &(sectionInfo [k].title), resources, XtNumber (resources), NULL, 0);
	CutSpaces (sectionInfo [k].title);			        /* Remove trailing spaces, if any */
	/* printf ("Title       of section %d_%d = Index %d: %s.\n", i, j, k, sectionInfo [k].title); */
	
	resources [0].resource_class = "SectionFilename";		/* Retrieve section filenames */
	resources [0].resource_type = XtRString;			
	resources [0].resource_size = sizeof (String);
	resources [0].resource_offset = 0;
	resources [0].default_type = XtRImmediate;
	resources [0].default_addr = (caddr_t) "";
	sectionInfo [k].filename = "Noch nix...";
	sprintf (resource_name,"section_%d_%d_filename", i,j);	/* Construct resource name, e.g. "section_2_0_filename" */
	resources [0].resource_name = resource_name;
	sectionInfo [k].filename = (String) XtMalloc (80*sizeof(char));
	sectionInfo [k].filename = "Noch nix...";
	XtGetApplicationResources (toplevel, &(sectionInfo [k].filename), resources, XtNumber (resources), NULL, 0);
	CutSpaces (sectionInfo [k].filename);			/* Remove trailing spaces, if any */
	/* printf ("Filename    of section %d_%d = Index %d: %s.\n", i, j, k, sectionInfo [k].filename); */
	
	resources [0].resource_class = "HasProgram";	/* Retrieve whether section has example program & resource */
	resources [0].resource_type = XtRBoolean;		
	resources [0].resource_size = sizeof (Boolean);
	resources [0].resource_offset = 0;
	resources [0].default_type = XtRImmediate;
	resources [0].default_addr = (caddr_t) FALSE;
	sectionInfo [k].hasProgram = FALSE;
	sprintf (resource_name,"section_%d_%d_hasProgram", i, j);  /* Construct resource name, e.g. "section_2_0_hasProgram" */
	resources [0].resource_name = resource_name;
	XtGetApplicationResources (toplevel, &(sectionInfo [k].hasProgram), resources, XtNumber (resources), NULL, 0);
	/* printf ("Has Program in Section %d_%d = Index %d: %u.\n", i, j, k, (Cardinal) sectionInfo [k].hasProgram); */
    };

}


void InitIndex (Widget selectionbox)

{
  XmString *entryStrings;
  Cardinal i;

  ReadIndexFile ();
  entryStrings = (XmString *) XtMalloc (sizeof (XmString) * numberOfIndexEntries);
  for (i=0; i<numberOfIndexEntries; i++)
    entryStrings [i] = XmStringCreate (indexEntries [i].keyword, XmSTRING_DEFAULT_CHARSET);
  XtVaSetValues (selectionbox,
		 XmNlistItems, entryStrings,
		 XmNlistItemCount, numberOfIndexEntries,
		 XmNmustMatch, True,
		 NULL);
  for (i=0; i<numberOfIndexEntries; i++)
    XmStringFree (entryStrings [i]);
  XtFree ((char *) entryStrings);
}
		 
