/***************************************************************************

     File: sectionio.c
 Contents: Input/Output functions to read sections and
           to save & restore resource files
  Part of: The Xmtutor application
   Author: Jan Borchers (job@ira.uka.de)

  $Source: /home/mit1/job/distribution/Xmtut/RCS/sectionio.c,v $
$Revision: 1.4 $
    $Date: 1994/10/19 10:29:10 $
   $State: Exp $
  $Locker:  $

$Log: sectionio.c,v $
 * Revision 1.4  1994/10/19  10:29:10  job
 * Increased allocated memory for strings in XtMalloc() calls by 1 byte
 * to make xmtutor work under Linux, too.
 *
 * Revision 1.3  1994/10/17  18:10:01  job
 * First revision under version control.
 *

***************************************************************************/


/**************************************************************************/
/*                                                                        */
/* COPYRIGHT NOTICE:                                                      */
/* -----------------                                                      */
/* All parts of the Xmtutor application and the sample Motif tutorial are */
/* Copyright (C) 1993,1994 Jan Borchers.                                  */
/*                                                                        */
/**************************************************************************/


#define INDEX_FILENAME "indexabc.txt"				/* Name of index file */

#include <stdio.h>						/* for file functions & debugging printf */
#include <string.h>						/* String handling */
#include <ctype.h>						/* toupper() */
#include <fcntl.h>						/* File handling */
#include <sys/stat.h>						/* INode information on files */
#include <sys/types.h>						/* Type definitions for stat() */

#include <X11/Intrinsic.h>					/* Xt structures & variables */
#include <X11/StringDefs.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>

#include "IMPORT.H"
#include "widgets.h"

#include "EXPORT.H"
#include "sectionio.h"						/* Interface declarations */



static String FilenameStart (void)				/* Returns start of filename incl. path */
                                                                /* e.g. "XMTUTOR_CONTENTSPATH/DisplayWidgets" */
								/* XMTUTOR_CONTENTSPATH is defined in makefile for xmtutor */
{
  String directory, filename, result;
  int i;
  
  directory = chapterInfo [currentChapter].directory;
  filename  = sectionInfo [chapterInfo [currentChapter].sectionOffset + currentSection].filename;

  i = strlen ("abc/def");
  
  result = (String) XtMalloc ((strlen (XMTUTOR_CONTENTSPATH) + 1 + strlen (directory) + strlen (filename) + 1) * sizeof (char));
  strcpy (result, XMTUTOR_CONTENTSPATH);
  strcat (result, "/");
  strcat (result, directory);
  strcat (result, filename);
  return result;						/* Return pointer to the start of the filename incl. path */
}


static String ExtendedFileName (String extension)		/* Get filenamestart and append extension, return result. */

{
  String filenamestart, result;
  
  filenamestart = FilenameStart ();
  result = (String) XtMalloc ((strlen (filenamestart) + strlen (extension) + 1) * sizeof (char));
  strcpy (result, filenamestart);
  strcat (result, extension);
  return result;
}


static String TextName (void)					/* Return filename for tutorial text file, */
								/* e.g. "XMTUTOR_CONTENTSPATH/DisplayWidgets/hello.txt" */
{
  return ExtendedFileName (".txt");
}


static String SourceCodeName (void)				/* Construct filename for program source file, */
								/* e.g. "XMTUTOR_CONTENTSPATH/DisplayWidgets/hello.c" */
{
  return ExtendedFileName (".c");
}


static String OriginalResourceName (void)			/* Construct filename for original (unchanged) */
								/* resource file to be read, */
{								/* e.g. "XMTUTOR_CONTENTSPATH/DisplayWidgets/Hello" */
  String result;
  char *cp;

  result = ExtendedFileName ("");				/* result="XMTUTOR_CONTENTSPATH/DisplayWidgets/hello"*/
  cp = strrchr (result,'/');					
  cp++;								/* cp points to first char after last "/" */
  *cp = toupper (*cp);						/* Capitalize first letter of program name */
  return result;
}


String UserResourceName (void)					/* Construct filename for (changed) resource file */
								/* to be saved & used by example program, */ 
{								/* e.g. "XMTUTOR_USERRESOURCEPATH/Hello" */
  String filename, result;					
  char *cp;
  
  filename = XtNewString(sectionInfo[chapterInfo[currentChapter].sectionOffset + currentSection].filename); /*eg hello */
  result = (String) XtMalloc ((strlen (XMTUTOR_USERRESOURCEPATH) + 1 + strlen (filename) + 1) * sizeof (char));
  strcpy (result, XMTUTOR_USERRESOURCEPATH);			/* result=XMTUTOR_USERRESOURCEPATH  */
  strcat (result, "/");						/* result=XMTUTOR_USERRESOURCEPATH/ */
  strcat (result, filename);					/* result=XMTUTOR_USERRESOURCEPATH/hello (e.g.) */
  cp = strrchr (result,'/');					
  cp++;								/* cp points to first char after last "/" */
  *cp = toupper (*cp);						/* Capitalize first letter of program name */
  XtFree ((char *) filename);
  return result;						/* result=XMTUTOR_USERRESOURCEPATH/Hello etc. */
}


String ExecName (void)						/* Construct filename for executable program file, */
								/* eg "XMTUTOR_CONTENTSPATH/DisplayWidgets/hello" */
								/* This routine is needed by runio.c */
{
  return ExtendedFileName ("");
}


static void ReadFile (String filename, String *filebufferptr)	/* Reads <filename> into <*filebufferptr> */
								/* Start address of filebuffer may be changed! */
{
  FILE *filepointer;						/* Pointer to opened file */
  Cardinal  filelength;						/* Length of file in bytes */
  struct stat statbuffer;					/* Stores INode information */

  filepointer = fopen (filename, "r");				/* Open file for reading */
  stat (filename, &statbuffer);					/* Fill statbuffer with information */
  filelength = statbuffer.st_size;				/* Get file length */
  *filebufferptr = (String) XtMalloc (sizeof(char)*(filelength+1)); /* Allocate memory */
  fread (*filebufferptr, sizeof (char), filelength, filepointer);   /* Read in file */
  *(*filebufferptr+filelength) = '\0';				/* Make filebuffer end with a NULL character */
  fclose (filepointer);						/* Close file */
}


static void WriteFile (String filename, String filebuffer)	/* Writes <filebuffer> to <filename> */

{
  FILE *filepointer;						/* Pointer to opened file */
  int  filelength;						/* Length of file in bytes */

  filepointer = fopen (filename, "w");				/* Open file for writing */
  filelength = strlen (filebuffer);				/* Get file length */
  fwrite (filebuffer, sizeof (char), filelength, filepointer);	/* Write file (without last NULL character)*/
  fclose (filepointer);						/* Close it */
}


void ReadIndexFile (void)						/* Read index file */

{
  FILE *filepointer;
  String filename;
  Cardinal i;
  char line [80], key [80];
  Cardinal chap, sect;
  
  filename = (String) XtMalloc ((strlen (XMTUTOR_CONTENTSPATH) + 1 + strlen (INDEX_FILENAME) + 1) * sizeof (char));
  sprintf (filename, "%s/%s", XMTUTOR_CONTENTSPATH, INDEX_FILENAME);
  filepointer = fopen (filename, "r");
  i = 0;
  while (fgets (line, 80, filepointer) != NULL)
    {
      if ((strlen (line) >= strlen ("A:0 0")) && (strchr(line, ':') != NULL))			       /* Skip lines that are empty or too short */
	{
	  strncpy (key, line, strcspn (line, ":"));		       /* Copy keyword up to colon */
	  key [strcspn (line, ":")] = '\0';			       /* End with '\0' */
	  sscanf (strchr (line, ':'), ":%u %u", &chap, &sect);	       /* Read chapter & section number after colon */
	  
	  indexEntries [i].keyword = XtNewString (key);
	  indexEntries [i].chapter = chap;
	  indexEntries [i].section = sect;
	  i++;
	}
    }
  numberOfIndexEntries = i;
  fclose (filepointer);
  XtFree ((char *) filename);
}


void UpdateInfoLine (void)					/* Updates info line after section change */

{
  char temp_string [80];
  
  XmString chaptertext, sectiontext;

  XtResource resources [] =					/* Get the prefix to display, eg "Chapter",from resource file*/
    {
      {XmNlabelString, XmCXmString, XmRXmString,
	 sizeof (XmString), (Cardinal) 0, XtRImmediate, (caddr_t) 0}
    };
  
  XtGetApplicationResources (chapterdisplay, &chaptertext, resources, XtNumber (resources), NULL, 0);
  sprintf (temp_string, " %u: %s", currentChapter, chapterInfo[currentChapter].title);
  chaptertext = XmStringConcat (chaptertext, XmStringCreate (temp_string, XmSTRING_DEFAULT_CHARSET));
  XtVaSetValues (chapterdisplay,
		 XmNlabelString, chaptertext,
		 NULL);

  sprintf (temp_string, "%s", sectionInfo[chapterInfo[currentChapter].sectionOffset + currentSection].title);
  sectiontext = XmStringCreate (temp_string, XmSTRING_DEFAULT_CHARSET);
  XtVaSetValues (sectiondisplay,
		 XmNlabelString, sectiontext,
		 NULL);

}


void SaveResource (void)

{
  char *resourcebuffer = NULL;
  
  if (sectionInfo [chapterInfo [currentChapter].sectionOffset + currentSection].hasProgram)  /* If section has a program: */
    {
      resourcebuffer = (String) XmTextGetString (resourcewindow);
      WriteFile (UserResourceName (), resourcebuffer);
      XtFree (resourcebuffer);
    }
}


void RestoreResource (void)					/* Restores original version of a resource file */

{
  char *resourcebuffer = NULL;
  
  if (sectionInfo [chapterInfo [currentChapter].sectionOffset + currentSection].hasProgram)  /* If section has a program: */
    {
      ReadFile (OriginalResourceName (), &resourcebuffer);
      XmTextSetString (resourcewindow, resourcebuffer);
      XtFree (resourcebuffer);
    }
}


void ReadSection (void)						/* Reads in a section */

{
  char *textbuffer, *programbuffer, *resourcebuffer;

  textbuffer     = NULL;					/* Initialize buffers to "empty" */
  programbuffer  = NULL;
  resourcebuffer = NULL;

  ReadFile (TextName (), &textbuffer);				/* Read in text file */
  if (sectionInfo [chapterInfo [currentChapter].sectionOffset + currentSection].hasProgram)  /* If section has a program: */
    {
      ReadFile (SourceCodeName (), &programbuffer);		/* Read in program C source code file */
      ReadFile (UserResourceName (), &resourcebuffer);		/* Read in (possibly edited) resource file */
      XmTextSetEditable (resourcewindow, TRUE);			/* Make resource window editable only if */
								/* resource file exists */
      XtSetSensitive (restorebutton, TRUE);                     /* "Restore Resource" button ditto. */
      XtSetSensitive (runcascade, TRUE);			/* "Run Example" button ditto. */
    }
  else
    {
      XmTextSetEditable (resourcewindow, FALSE);
      XtSetSensitive (restorebutton, FALSE);
      XtSetSensitive (runcascade, FALSE);
    }
  UpdateInfoLine ();						/* Updates info line after section change */
								/* Set sensitivity of various widgets according to position */
  XtSetSensitive (nextsectionbutton, (!(currentChapter == numberOfChapters-1 &&
					currentSection == chapterInfo[currentChapter].numberOfSections-1)));
  XtSetSensitive (previoussectionbutton, (!(currentChapter == 0 && currentSection == 0)));
  
  XmTextSetString (textwindow, textbuffer);			/* Load new buffers (possibly empty) into display widgets */
  XmTextSetString (programwindow, programbuffer);
  XmTextSetString (resourcewindow, resourcebuffer);
  XtFree (textbuffer);						/* Free buffers */
  XtFree (programbuffer);
  XtFree (resourcebuffer);
}
